<?php

namespace plugin\ticket\controller;

use plugin\ticket\model\TicketDept;
use think\admin\Controller;
use think\admin\helper\QueryHelper;
use think\admin\model\SystemUser;

/**
 * 部门管理控制器
 */
class Dept extends Controller
{
    /**
     * 部门列表页面
     * 显示部门管理的表格数据，支持搜索和筛选。
     * @auth true
     * @menu true
     */
    public function index()
    {
        $this->title = '部门管理';
        TicketDept::mQuery()->layTable(function () {

        }, function (QueryHelper $query) {
            $query->like('name')->with('fz_user')->equal('status');
        });
    }

    /**
     * 添加部门页面
     * 提供表单用于添加新部门，并加载用户列表以供选择负责人。
     * @auth true
     * @menu true
     */
    public function add()
    {
        $this->title = '添加部门';
        $this->users = SystemUser::query()->where(['status' => 1])->order('sort desc,id desc')->select();
        TicketDept::mForm('form');
    }

    /**
     * 编辑部门页面
     * 提供表单用于编辑现有部门信息，并加载用户列表以供选择负责人。
     * @auth true
     * @menu true
     */
    public function edit()
    {
        $this->title = '编辑部门';
        $this->users = SystemUser::query()->where(['status' => 1])->order('sort desc,id desc')->select();
        TicketDept::mForm('form');
    }

    /**
     * 删除部门
     * 根据提交的 ID 删除指定部门。
     * @auth true
     * @menu true
     */
    public function remove()
    {
        $this->_applyFormToken();
        TicketDept::mDelete('id');
    }

    /**
     * 更新部门状态
     * 根据提交的 ID 和状态值更新部门的状态。
     * @auth true
     * @menu true
     */
    public function status()
    {
        TicketDept::mSave($this->_vali([
            'id.require' => '部门ID不能为空',
            'status.in:0,1'  => '状态值范围异常！',
            'status.require' => '状态值不能为空！',
        ]), 'id');
    }
}