<?php

namespace plugin\ticket\controller;

use PhpOffice\PhpSpreadsheet\IOFactory;
use plugin\inspection\model\InspectionStaff;
use plugin\ticket\model\ApprovalInstance;
use plugin\ticket\model\ApprovalProcess;
use plugin\ticket\model\ApprovalStep;
use plugin\ticket\model\TicketReply;
use plugin\ticket\model\TicketTicket;
use plugin\ticket\model\TicketTicketOuter;
use plugin\ticket\model\TicketType;
use think\admin\Controller;
use think\admin\helper\QueryHelper;
use think\admin\model\SystemUser;
use think\exception\HttpResponseException;

/**
 * 外部工单管理
 */
class TicketOuter extends Controller
{
    /**
     * 工单列表
     * @auth true
     * @menu true
     * @return void
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public function index()
    {
        $this->title = '工单中心';
        $this->type_list = TicketType::getList();
        $this->user_id = $this->request->session('user')['id'];
        TicketTicketOuter::mQuery()->layTable(function () {

        }, function (QueryHelper $query) {
            $query->like(['title|content|contact_name|ticket_address|contact_phone#keyword'])
                ->dateBetween(['create_at'])
                ->equal(['status', 'type_id']);
            $query->append(['imgs_arr', 'source_type_name', 'type_name', 'last_reply'])->with(['user_shares', 'views', 'repairs', 'verifys', 'view_process', 'repair_process', 'verify_process']);
        });
    }

    /**
     * 查看工单
     * @auth true
     * @menu true
     * @return void
     */
    public function detail()
    {
        $this->title = '工单详情';
        ['id' => $id] = $this->_vali(['id.require' => '请指定工单ID！']);
        $this->vo = TicketTicketOuter::mk()->with(['user', 'type'])->append(['imgs_arr', 'type_name'])->find($id);
        $this->ticket = $this->vo;
        $process = ApprovalProcess::where('type', '=', 'GDSH')->order('id', 'asc')->findOrEmpty();
        $instance = ApprovalInstance::query()->with(['steps.approver', 'logs'])->where('process_id', '=', $process->id)->where('oid', '=', $id)->findOrEmpty();
        $this->instance = $instance;
        if ($instance->isEmpty()) {
            $this->step_index = -1;
        } else {
            $this->step_index = $instance->current_step;
        }
        $this->fetch();
    }

    public function reply()
    {
        $this->title = '回复工单';
        [
            'ticket_id'=>$ticket_id,
            'id'=>$id
        ] = $this->_vali([
            'ticket_id.require'=>'请指定工单ID！',
            'id.require'=>'请指定回复ID！'
        ]);
        $this->vo = TicketTicket::mk()->with(['user', 'type'])->append(['imgs_arr', 'type_name'])->find($ticket_id);
        TicketReply::mForm('reply');
    }

    public function _reply_form_filter(&$data)
    {
        if ($this->request->isPost()) {
            $this->_applyFormToken();
            $data['type'] = 1;
            $adminInfo = $this->request->session('user');
            $data['user_id'] = $adminInfo['id'];
            $data['username'] = $adminInfo['nickname'];
            $data['contact'] = $adminInfo['contact_phone'];
        }
    }

    public function my() {
        $this->title = '待我审核';
        $this->type_list = TicketType::getList();
        $this->user_id = $this->request->session('user')['id'];
        TicketTicket::mQuery()->layTable(function () {

        }, function (QueryHelper $query) {
            $query->like(['title|content|contact_name|ticket_address|contact_phone#keyword'])
                ->dateBetween(['create_at'])
                ->equal(['status', 'type_id']);
            $query->whereIn("id", ApprovalStep::query()->where(['approver_id' => $this->user_id, 'status' => 0, 'approver_type' => 1, 'instance.status' => 0])->field("oid")->select())
                ->append(['imgs_arr', 'source_type_name', 'type_name', 'last_reply'])->with('approval');
        });
    }

    /**
     * @return void
     * @auth true
     */
    public function import() {
        $file = $this->app->request->post('file');
        if (!$file) {
            // 创建一个临时Excel模板用于下载
            $this->redirect('/static/excel/ticket_import_template.xlsx');
        };
        $file = '.' . str_replace($this->app->request->domain(), '', $file);
        //表格字段对应
        $fields = [
            'A' => 'title',
            'B' => 'type_id',
            'C' => 'content',
            'D' => 'contact_name',
            'E' => 'contact_phone',
            'F' => 'contact_address',
            'G' => 'ticket_address',
        ];
        //加载文件
        $spreadsheet = IOFactory::load($file);

        $sheet = $spreadsheet->getActiveSheet();               // 获取表格
        $highestRow = $sheet->getHighestRow();                 // 取得总行数
        $sheetData = [];
        for ($row = 2; $row <= $highestRow; $row++) {          // $row表示从第几行开始读取
            foreach ($fields as $cell => $field) {
                $value = $sheet->getCell($cell . $row)->getValue();
                $value = trim($value);
                $sheetData[$row][$field] = $value;
            }
        }
        $sheetData = array_values($sheetData);
        foreach ($sheetData as $key => $value) {
            if (!empty($value['type_id'])) {
                $type = TicketType::query()->where(['id' => $value['type_id']])->find();
                if (empty($type)) {
                    $this->error('工单类型【'.$value['type_id'].'】不存在');
                } else {
                    $sheetData[$key]['type_id'] = $type['id'];
                }
            }
            $sheetData[$key]['create_at'] = date('Y-m-d H:i:s');
            $sheetData[$key]['user_id'] = $this->request->session('user')['id'];
            $sheetData[$key]['status'] = 0;
        }
        TicketTicket::mk()->saveAll($sheetData);
        $this->success("成功");
    }

    public function comment()
    {
        $data = $this->_vali([
            'id.require'=>'请指定工单ID！',
        ]);
        $ticket = TicketTicket::query()->with(['user_shares'])->where('id', '=', $data['id'])->findOrEmpty();
        if ($ticket->isEmpty()) {
            $this->error('未找到工单信息');
        }
        if (!$ticket->user_shares || sizeof($ticket->user_shares) == 0) {
            $this->error("该工单无需评论");
        }
        if ($this->request->isPost()) {
            $adminInfo = $this->request->session('user');
            $comment_data = $this->_vali([
                'content.require'=>'请输入评论内容！',
            ]);
            TicketTicket::mk()->startTrans();
            try {
                foreach ($ticket->user_shares as $share) {
                    $share->logs()->save([
                        'op_name' => $adminInfo["username"],
                        'content' => $comment_data['content'],
                        'result' => "后台用户反馈",
                    ]);
                }
                TicketTicket::mk()->commit();
            } catch (\Exception $e) {
                TicketTicket::mk()->rollback();
                $this->error("保存失败");
            }
        } else {
            $this->vo = $ticket;
            $this->ticket = $ticket;
            $this->fetch();
        }
    }

}